@@ -58,6 +58,19 @@ class UnusableMessageError(Exception):
     """
 
 
+class ReceiverError(Exception):
+    """
+    Indicates that one or more receivers of a signal raised an exception when called.
+    """
+
+    def __init__(self, message: str, causes: list):
+        """
+        Create ReceiverError with a message and a list of exceptions returned by receivers.
+        """
+        super().__init__(message)
+        self.causes = causes  # just used for testing
+
+
 class KafkaEventConsumer:
     """
     Construct consumer for the given topic, group, and signal. The consumer can then
@@ -159,7 +172,7 @@ class KafkaEventConsumer:
                     # to commit all this consumer's current offset across all partitions since we only process one
                     # message at a time, but limit it to just the offset/partition of the specified message
                     # to be super safe
-                    self.consumer.commit(message=msg, asynchronous=False)
+                    self.consumer.commit(message=msg)
         finally:
             self.consumer.close()
 
@@ -203,13 +216,20 @@ class KafkaEventConsumer:
             )
 
         send_results = self.signal.send_event(**msg.value())
-        self.report_receiver_errors(send_results)
+        # Raise an exception if any receivers errored out. This allows logging of the receivers
+        # along with partition, offset, etc. in record_event_consuming_error. Hopefully the
+        # receiver code is idempotent and we can just replay any messages that were involved.
+        self._check_receiver_results(send_results)
 
-    def report_receiver_errors(self, send_results):
+    def _check_receiver_results(self, send_results: list):
         """
-        Takes the list of (receiver, response) pairs and logs a warning if any are errors.
+        Raises exception if any of the receivers produced an exception.
+
+        Arguments:
+            send_results: Output of ``send_events``, a list of ``(receiver, response)`` tuples.
         """
-        error_list = []
+        error_descriptions = []
+        errors = []
         for receiver, response in send_results:
             if not isinstance(response, BaseException):
                 continue
@@ -221,12 +241,16 @@ class KafkaEventConsumer:
             except AttributeError:
                 receiver_name = str(receiver)
 
-            error_list.append(f"{receiver_name}={response!r}")
+            # The stack traces are already logged by django.dispatcher, so just the error message is fine.
+            error_descriptions.append(f"{receiver_name}={response!r}")
+            errors.append(response)
 
-        if len(error_list) > 0:
-            logger.warning(
-                f"{len(error_list)} receiver(s) out of {len(send_results)} "
-                f"produced errors when handling signal {self.signal}: {', '.join(error_list)}"
+        if len(error_descriptions) > 0:
+            raise ReceiverError(
+                f"{len(error_descriptions)} receiver(s) out of {len(send_results)} "
+                "produced errors (stack trace elsewhere in logs) "
+                f"when handling signal {self.signal}: {', '.join(error_descriptions)}",
+                errors
             )
 
     def record_event_consuming_error(self, run_context, error, maybe_event):
@@ -271,7 +295,8 @@ class ConsumeEventsCommand(BaseCommand):
     help = """
     Consume messages of specified signal type from a Kafka topic and send their data to that signal.
 
-    example:
+    Example::
+
         python3 manage.py cms consume_events -t user-login -g user-activity-service \
             -s org.openedx.learning.auth.session.login.completed.v1
     """
